"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const controller_1 = require("@curveball/controller");
const http_errors_1 = require("@curveball/http-errors");
const privilegeService = require("../../privilege/service");
const principalService = require("../../principal/service");
const html_1 = require("../formats/html");
const crypto_1 = require("../../crypto");
class CreateAppController extends controller_1.default {
    async get(ctx) {
        if (!await privilegeService.hasPrivilege(ctx, 'admin')) {
            throw new http_errors_1.Forbidden('Only users with the "admin" privilege can create new users');
        }
        ctx.response.type = 'text/html';
        ctx.response.body = (0, html_1.createAppForm)({
            csrfToken: await ctx.getCsrf(),
            msg: ctx.query.msg,
            error: ctx.query.error,
            nickname: ctx.query.nickname,
            url: ctx.query.url,
            clientId: ctx.query.clientId,
            allowedGrantTypes: ctx.query.allowedGrantTypes,
            redirectUris: ctx.query.redirectUris,
            requirePkce: ctx.query.requirePkce
        });
    }
    async post(ctx) {
        ctx.request.validate('https://curveballjs.org/schemas/a12nserver/app-new-form.json');
        if (!await privilegeService.hasPrivilege(ctx, 'admin')) {
            throw new http_errors_1.Forbidden('Only users with the "admin" privilege can create new apps');
        }
        const nickname = ctx.request.body.nickname;
        const identity = ctx.request.body.url || (0, crypto_1.uuidUrn)();
        const newApp = await principalService.save({
            type: 'app',
            nickname,
            active: true,
            identity,
            createdAt: new Date(),
            modifiedAt: new Date(),
        });
        ctx.response.status = 303;
        let newLocation = newApp.href;
        if (ctx.request.body.clientId || ctx.request.body.allowedGrantTypes || ctx.request.body.redirectUris || ctx.request.body.requirePkce) {
            const params = {
                ...(ctx.request.body.clientId && {
                    clientId: ctx.request.body.clientId
                }),
                ...(ctx.request.body.allowedGrantTypes && {
                    allowedGrantTypes: ctx.request.body.allowedGrantTypes
                }),
                ...(ctx.request.body.redirectUris && {
                    redirectUris: ctx.request.body.redirectUris
                }),
                ...(ctx.request.body.requirePkce && {
                    requirePkce: ctx.request.body.requirePkce
                }),
            };
            newLocation = `${newLocation}/client/new?${new URLSearchParams(params)}`;
        }
        ctx.response.status = 303;
        ctx.response.headers.set('Location', newLocation);
    }
}
exports.default = new CreateAppController();
//# sourceMappingURL=new.js.map