import { NewPrincipal, PrincipalStats } from './types';
import { Principal, PrincipalType, User, Group, App, BasePrincipal } from '../types';
import { PrincipalsRecord } from 'knex/types/tables';
export declare class InactivePrincipal extends Error {
}
export declare const fieldNames: Array<keyof PrincipalsRecord>;
export declare function findAll(type: 'user'): Promise<User[]>;
export declare function findAll(type: 'group'): Promise<Group[]>;
export declare function findAll(type: 'app'): Promise<App[]>;
export declare function findAll(): Promise<Principal[]>;
export declare function getPrincipalStats(): Promise<PrincipalStats>;
export declare function findById(id: number, type: 'user'): Promise<User>;
export declare function findById(id: number, type: 'group'): Promise<Group>;
export declare function findById(id: number, type: 'app'): Promise<App>;
export declare function findById(id: number): Promise<Principal>;
export declare function findByExternalId(externalId: string, type: 'user'): Promise<User>;
export declare function findByExternalId(externalId: string, type: 'group'): Promise<Group>;
export declare function findByExternalId(externalId: string, type: 'app'): Promise<App>;
export declare function findByExternalId(externalId: string): Promise<Principal>;
/**
 * Find multiple principals.
 *
 * This function returns the principals as a map, index by their id.
 * This is a helper function typically used by other services to find large numbers
 * of joined principals from other tables fast.
 *
 * If any of the ids in the list is duplicated, they are de-duplicated here.
 * if any of the provided ids are not found, this function will error.
 */
export declare function findMany(ids: number[]): Promise<Map<number, Principal>>;
/**
 * Returns true if more than 1 principal exists in the system.
 *
 * If there are 0 principals, this puts a12nserver in setup mode, which allows a
 * person to create the first user in the system, automatically activate it
 * and make them an admin.
 */
export declare function hasPrincipals(): Promise<boolean>;
export declare function findByIdentity(identity: string): Promise<Principal>;
/**
 * Finds a principal by its href'.
 *
 * This can be a string like /user/1, or a full url.
 * It can also be the uri listed in the 'identity' field.
 */
export declare function findByHref(href: string): Promise<Principal>;
export declare function save<T extends PrincipalType>(principal: BasePrincipal<T> | NewPrincipal<T>): Promise<BasePrincipal<T>>;
export declare function recordToModel(user: PrincipalsRecord): Principal;
export declare function isIdentityValid(identity: string): boolean;
export declare function getPathName(href: string): string;
