"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const controller_1 = require("@curveball/controller");
const http_errors_1 = require("@curveball/http-errors");
const privilegeService = require("../../privilege/service");
const hal = require("../formats/hal");
const principalService = require("../../principal/service");
class UserEditPrivilegesController extends controller_1.default {
    async get(ctx) {
        const user = await principalService.findByExternalId(ctx.params.id);
        const userPrivileges = await privilegeService.getImmediatePrivilegesForPrincipal(user);
        const privileges = await privilegeService.findPrivileges();
        await privilegeService.hasPrivilege(ctx, 'admin');
        ctx.response.body = hal.editPrivileges(user, userPrivileges, privileges.map(privilege => privilege.privilege));
    }
    async post(ctx) {
        const { policyBody } = ctx.request.body;
        const principal = await principalService.findByExternalId(ctx.params.id);
        await privilegeService.hasPrivilege(ctx, 'admin');
        try {
            const policy = JSON.parse(policyBody);
            await privilegeService.replacePrivilegeForUser(principal, policy);
        }
        catch (err) {
            throw new http_errors_1.BadRequest(err);
        }
        ctx.redirect(303, principal.href);
    }
    async patch(ctx) {
        ctx.request.validate('https://curveballjs.org/schemas/a12nserver/principal-patch-privilege.json');
        const principal = await principalService.findByExternalId(ctx.params.id);
        await privilegeService.hasPrivilege(ctx, 'admin');
        await privilegeService.addPrivilegeForUser(principal, ctx.request.body.privilege, ctx.request.body.resource);
        ctx.status = 200;
        ctx.response.body = {
            _links: {
                principal: {
                    href: principal.href,
                    title: principal.nickname,
                },
                up: {
                    href: `${principal.href}/edit/privileges`,
                    title: 'Back to privileges',
                }
            },
            title: 'Privileges updated',
        };
    }
}
exports.default = new UserEditPrivilegesController();
//# sourceMappingURL=privileges.js.map